<?php

App::uses('BuzzBookingsAppController', 'BuzzBookings.Controller');
App::uses('CustomerAddress', 'BuzzCustomers.Model');

class BookingsController extends BuzzBookingsAppController {

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'packages',
			'date',
			'extras',
			'ajax_voucher',
			'ajax_dates',
			'ajax_times',
			'checkout'
		));
	}

/**
 * Bookings landing page - will mostly get bypassed by homepage/activity pages
 *
 * @return void
 */
	public function index() {
		$Booking = $this->{$this->modelClass};

		if (!empty($this->request->data)) {

			if (!empty($this->request->data['Booking']['activity_id'])) {
				// Forward to the correct packages step.
				return $this->redirect([
					'action' => 'packages',
					$this->request->data['Booking']['activity_id']
				]);
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please select an activity'),
					'flash_fail'
				);
			}

		}

		$activities = $Booking->Activity->getListed();
		$activities = Hash::combine($activities, '{n}.Activity.id', '{n}.Activity.name');

		if (count($activities) === 1) {
			return $this->redirect([
				'action' => 'packages'
			]);
		} elseif (empty($activities)) {
			// No activities have been setup or created so throw an exception
			throw new NotFoundException();
		}

		$this->set(compact('activities'));
		$this->view = 'BuzzBookings./Bookings/index';

		return;
	}

/**
 * Packages step - add packages and vouchers to a booking
 *
 * To jump to the packages step for a specific activity the activity ID can be
 * passed; otherwise the activity ID is set on the booking.
 *
 * To preselect a specific package a package ID needs to be passed.
 *
 * @param int $activityId Activity ID
 * @param int $packageId Optional activity package ID
 * @return void
 */
	public function packages($activityId = null, $packageId = null) {
		$Booking = $this->{$this->modelClass};

		// Process form data.
		if (!empty($this->request->data)) {

			$result = $Booking->createBasket($activityId, $this->request->data['BookingItem']);
			if ($result === true) {
				return $this->redirect(['action' => 'date']);
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please select a package'),
					'flash_fail'
				);
			}

		}

		// Get the basket.
		$basket = $Booking->getBasket();

		if (empty($activityId)) {

			if (!empty($basket['Booking']['activity_id'])) {
				// Use the basket's activity.
				$activityId = $basket['Booking']['activity_id'];
			} else {
				// Check if the site is configured to only have one listed
				// bookable activity. This is needed to prevent a redirect
				// loop with this and the previous step.
				$activityId = $Booking->Activity->onlyBookableActivity();
			}

		}

		// If the activity ID hasn't been set then redirect to the previous
		// step.
		if (empty($activityId)) {
			return $this->redirect([
				'action' => 'index'
			]);
		} else {
			// Set the booking's activity ID.
			$this->request->data['Booking']['activity_id'] = $activityId;
		}

		// Get the packages and activity details for generating the form.
		$data = $Booking->Activity->getBookableActivity($activityId);

		if (empty($data)) {
			throw new NotFoundException();
		}

		if (
			!empty($basket) && (int)$basket['Booking']['activity_id'] !== (int)$activityId
		) {
			// Clear the basket as we're either changing the activity being booked so
			// existing items are no longer valid.
			$Booking->clearBasket(false);
			// We don't need to pass the basket to the view anymore as we're
			// essentially starting a fresh basket.
			$basket = [];
		} elseif (empty($this->request->data['BookingItem']) && !empty($basket['BookingItem'])) {
			// Populate the form with existing booking items.
			$this->request->data['BookingItem'] = Hash::combine($basket['BookingItem'], '{n}.activity_package_id', '{n}');
		}

		// If someone's followed a link to directly book a specific package
		// then set the quantity to 1.
		if (!empty($packageId) && empty($this->request->data['BookingItem'])) {
			$this->request->data['BookingItem'][$packageId]['quantity'] = 1;
		}

		$previousStep = ['action' => 'index'];
		// Check if this is a non-listed activity (i.e. we will need to return
		// the user to the activity page rather than allow them to select a
		// different activity).
		if (!empty($basket['Activity']['listed']) && (bool)$basket['Activity']['listed'] === false) {
			$previousStep = ['controller' => 'activities', 'action' => 'view', $activityId];
		}

		$this->set(compact('data', 'basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/packages';

		return;
	}

/**
 * Date step - book a time and date for the booking
 *
 * @return void
 */
	public function date() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}

		$previousStep = ['action' => 'packages', $basket['Booking']['activity_id']];

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/date';

		return;
	}

	public function extras() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}

		$extras = $Booking->getExtras($basket);

		$previousStep = ['action' => 'date'];

		$this->set(compact('basket', 'extras', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/extras';

		return;
	}

/**
 * AJAX call for generating voucher fields.
 *
 * @param int $activityId Activity ID
 * @return void
 */
	public function ajax_voucher($activityId) {
		$Booking = $this->{$this->modelClass};

		$data = $Booking->getVouchers($activityId);

		// Build a simple array of voucher providers for convenience in the View.
		$voucherProviders = Hash::combine($data, '{n}.VoucherProvider.id', '{n}.VoucherProvider.name');
		asort($voucherProviders);

		$this->set(compact('data', 'voucherProviders'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/ajax_voucher';

		return;
	}

/**
 * AJAX call for generating the available times and dates for the current
 * booking.
 *
 * @return void
 */
	public function ajax_dates() {
		$Booking = $this->{$this->modelClass};

		// Get the available times and dates.
		$fromDate = date('Y-m-d');
		$toDate = date('Y-m-d', strtotime('+3 months'));
		$data = $Booking->getAvailableDates($fromDate, $toDate);

		if (!empty($data)) {
			// Work out the unavailable dates within our date range.
			$unavailableDates = $Booking->calculateUnavailableDates($data, $fromDate, $toDate);

			// Store the available dates/times in the session.
			$this->Session->write('BuzzBookings.availableDates', Hash::combine($data, '{n}.date', '{n}.times'));

			$this->set(compact('unavailableDates', 'fromDate', 'toDate'));
			$view = 'ajax_dates';
		} else {
			$view = 'ajax_dates_unavailable';
		}

		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/' . $view;

		return;
	}

/**
 * AJAX call for displaying available times.
 *
 * @param string $date
 * @return void
 */
	public function ajax_times($date) {
		$Booking = $this->{$this->modelClass};

		$data = $this->Session->read('BuzzBookings.availableDates.' . $date);

		$this->set(compact('data', 'date'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/ajax_times';

		return;
	}

/**
 * Checkout step
 *
 * @return void
 */
	public function checkout() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();

		if (!empty($this->request->data)) {

			$result = $Booking->addBillingDetails($this->request->data['CustomerAddress']);
			if ($result === true) {
				// Take payment.
				$this->_pay();
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please correct the errors below'),
					'flash_fail'
				);
			}

		} else {

			// Set the address field defaults (these are configured by the
			// CustomerAddress plugin to make it easier to override for each site).
			$this->request->data['CustomerAddress']['country_id'] = CustomerAddress::getDefaultCountry();
			$this->request->data['CustomerAddress']['us_state_id'] = CustomerAddress::getDefaultUsState();

		}

		// Get the basket.
		$basket = $Booking->getBasket();

		// Get gift voucher purchase conditions
		$this->loadModel('BuzzConditions.Condition');
		$conditions = $this->Condition->getConditions('Booking');

		$previousStep = ['action' => 'delivery'];

		$this->set(compact('basket', 'conditions', 'previousStep'));
		$this->_populateLookups();
		$this->view = 'BuzzBookings.checkout';

		return;
	}

/**
 * Processes the payment and triggers the completion of the payment on success
 *
 * @return void
 */
	protected function _pay() {
		$Booking = $this->{$this->modelClass};

		// Get the latest basket.
		$basket = $Booking->getBasket();

		if ($basket['Booking']['total_cost'] > 0) {

			$transaction = $this->Transactions->takePayment(
				// Return URLs
				array(
					'return' => Router::url(['action' => 'redsys_callback'], true),
					'cancel' => Router::url(['action' => 'checkout'], true)
				),
				// Calling record
				array(
					'model' => 'BuzzBookings.Booking',
					'model_id' => $basket['Booking']['id']
				),
				// Amount to be paid
				$basket['Booking']['total_cost'],
				// Items
				array(),
				// Extras
				array(
					'language' => Configure::read('Config.language')
				),
				0,
				Configure::read('BuzzPurchase.payment_gateway')
			);

			if (!empty($transaction)) {
				// Payment taken successfully, complete purchase.
				return $this->_completePurchase();
			} else {
				$this->Session->setFlash(
					__d(
						'buzz_bookings',
						'There was a problem processing your payment, please check your details and try again'
					),
					'flash_fail'
				);
			}

		} else {
			// Nothing to pay, complete purchase.
			return $this->_completePurchase();
		}


		return;
	}

/**
 * Complete the purchase (after payment received)
 *
 * @return void Redirects to confirmation page
 */
	protected function _completePurchase() {
		$Booking = $this->{$this->modelClass};

		$basketId = $Booking->getBasketId();

		if (empty($basketId)) {
			throw new ForbiddenException();
		}

		$this->Session->write('Booking.hash', $Booking->hashBasketId());

		$Booking->completePurchase(true);

		return $this->redirect([
			'action' => 'confirmation',
			$basketId
		]);
	}

/**
 * Confirmation page
 *
 * @param int $id Basket ID
 * @return void
 */
	public function confirmation($id) {
		$Booking = $this->{$this->modelClass};

		if ($Booking->hashBasketId($id) === $this->Session->read('Booking.hash')) {

			$this->Session->delete('Booking.hash');

			$basket = $Booking->getPurchase($id);

			if (Configure::check('BuzzBookings.confirmation_page_id') === true) {
				$this->assignPage(Configure::read('BuzzBookings.confirmation_page_id'));
			}

			$this->set(compact('basket'));
			$this->view = 'BuzzBookings.confirmation';

		} else {

			throw new ForbiddenException();

		}

		return;
	}

/**
 * Populate lookups for address fields.
 *
 * @return void
 */
	protected function _populateLookups() {
		$Bookings = $this->{$this->modelClass};

		$countries = ClassRegistry::init('BuzzCustomers.Country')->translatedList();
		$usStates = $Bookings->CustomerAddress->UsState->find('list');

		$this->set(compact('countries', 'usStates'));

		return;
	}

}
